#!/bin/sh -e

#
# Copyright (c) 2022 VMware, Inc. All Rights Reserved.
#

set -e -o pipefail

# Init Java options - ORG_JAVA_OPTS will keep the original value when the script was started for the first time
JAVA_OPTS="${ORG_JAVA_OPTS:=${JAVA_OPTS:- }}"
export ORG_JAVA_OPTS

# Allow custom props to be inserted
test -f /custom-conf1.sh && . /custom-conf1.sh

# remove options not supported in OpenJDK 11
JAVA_OPTS=$( <<< "$JAVA_OPTS" sed -e 's/-XX:+UseParNewGC//' )

# General Java options
JAVA_OPTS+=" -showversion"
JAVA_OPTS+=" -XX:+PrintCommandLineFlags"
JAVA_OPTS+=" -XX:+PrintStringTableStatistics"
JAVA_OPTS+=" -XX:ErrorFile=/var/log/error_java_%p.log"
JAVA_OPTS+=" -Xlog:gc*:file=/var/log/gc.log:time,uptime,level,tags:filecount=10,filesize=10m"

# memory options
# use parallel GC if no GC is added in JAVA_OPTS - the default G1 uses too much off-heap memory
if ! grep -E 'UseG1GC|UseConcMarkSweepGC|UseEpsilonGC|UseParallelGC|UseSerialGC|UseZGC' <<< "$JAVA_OPTS" > /dev/null
then
  echo "Setting GC to ParallelGC"
  JAVA_OPTS+=" -XX:+UseParallelGC"
fi

echo "Setting heap to ${JVM_HEAP:=1024m}"
JAVA_OPTS+=" -Xms$JVM_HEAP -Xmx$JVM_HEAP"

echo "Setting metaspace to ${JVM_METASPACE:=256m}"
JAVA_OPTS+=" -XX:MaxMetaspaceSize=$JVM_METASPACE"

HEAP_DUMP_PATH="/var/log/$( date '+%Y_%m_%d_%H_%M' )_heap_dump.hprof"

JAVA_OPTS+=" -Xss512K"
JAVA_OPTS+=" -XX:+HeapDumpOnOutOfMemoryError -XX:HeapDumpPath=$HEAP_DUMP_PATH"
JAVA_OPTS+=" -XX:+ExitOnOutOfMemoryError"
JAVA_OPTS+=" -XX:+UseStringDeduplication"


if [ -n "$DEBUG_PORT" ]; then
  echo "Enabling debugging on port: $DEBUG_PORT"
  JAVA_OPTS+=" -Xdebug -Xrunjdwp:transport=dt_socket,suspend=n,server=y,address=$DEBUG_PORT"
fi

if [ -n "$SERVER_PORT" ]; then
  JAVA_OPTS+=" -Dserver.port=$SERVER_PORT"
fi

if [ -n "$JMX_PORT" ]; then
  echo "Enabling JMX connection on port: $JMX_PORT"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.port=$JMX_PORT"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.rmi.port=$JMX_PORT"
  JAVA_OPTS+=" -Djava.rmi.server.hostname=${JMX_HOST:-127.0.0.1}"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.ssl=false"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.local.only=false"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.authenticate=false"
fi

if [ -n "$EXT_JAVA_OPTS" ]; then
  echo "External java options: $EXT_JAVA_OPTS"
  JAVA_OPTS+=" $EXT_JAVA_OPTS"
fi

# Add ingress/load-balancert/proxy certificates in the java keystore if such are mounted,
# configure FIPS properties
#
source /usr/local/bin/setup_security.sh

if [ -n "$JMX_PORT" ]; then
  echo "Enabling JMX connection on port: $JMX_PORT"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.port=$JMX_PORT"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.rmi.port=$JMX_PORT"
  JAVA_OPTS+=" -Djava.rmi.server.hostname=${JMX_HOST:-127.0.0.1}"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.ssl=false"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.local.only=false"
  JAVA_OPTS+=" -Dcom.sun.management.jmxremote.authenticate=false"
fi

if [ -n "$EXT_JAVA_OPTS" ]; then
  echo "External java options: $EXT_JAVA_OPTS"
  JAVA_OPTS+=" $EXT_JAVA_OPTS"
fi

#https://www.oracle.com/java/technologies/javase/jdk-11-relnote.html
#TLS 1.3 Half-Close Policy
JAVA_OPTS+=" -Djdk.tls.acknowledgeCloseNotify=true"

echo -e '\n\n==============\n\n\n'
status=0

##### Start the programs

touch /var/log/ensemble-app.log
tail -c 0 -F /var/log/ensemble-app.log &
TAILPID=$!

cd "$ENSEMBLE_HOME"

java \
    $JAVA_OPTS \
    -cp ${BCFIPS_JARS:+"$BCFIPS_JARS"}:. \
    -jar $ENSEMBLE_HOME/ensemble-app.jar \
    --spring.config.additional-location="$APPLICATION_CONFIG_FILE" \
    "$@" 2>&1 &

PID=$!

wait $PID || status=$?
echo "jvm exited with status code $status"
kill $TAILPID || true

exit "$status"
